@@ -1,10 +1,13 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import
-from __future__ import unicode_literals
+from __future__ import annotations
 
+import builtins
+import errno
 import io
 import os
+import socket
 import stat
+from tempfile import TemporaryDirectory
+from unittest import mock
 
 import pytest
 
@@ -14,6 +17,21 @@ from identify import identify
 def test_all_tags_includes_basic_ones():
     assert 'file' in identify.ALL_TAGS
     assert 'directory' in identify.ALL_TAGS
+    assert 'executable' in identify.ALL_TAGS
+    assert 'text' in identify.ALL_TAGS
+    assert 'socket' in identify.ALL_TAGS
+
+
+@pytest.mark.parametrize(
+    'tag_group',
+    (
+        identify.TYPE_TAGS,
+        identify.MODE_TAGS,
+        identify.ENCODING_TAGS,
+    ),
+)
+def test_all_tags_contains_all_groups(tag_group):
+    assert tag_group < identify.ALL_TAGS
 
 
 def test_all_tags_contains_each_type():
@@ -41,6 +59,17 @@ def test_tags_from_path_symlink(tmpdir):
     assert identify.tags_from_path(x.strpath) == {'symlink'}
 
 
+def test_tags_from_path_socket():
+    tmproot = '/tmp'  # short path avoids `OSError: AF_UNIX path too long`
+    with TemporaryDirectory(dir=tmproot) as tmpdir:
+        socket_path = os.path.join(tmpdir, 'socket')
+        with socket.socket(socket.AF_UNIX) as sock:
+            sock.bind(socket_path)
+            tags = identify.tags_from_path(socket_path)
+
+    assert tags == {'socket'}
+
+
 def test_tags_from_path_broken_symlink(tmpdir):
     x = tmpdir.join('foo')
     x.mksymlinkto(tmpdir.join('lol'))
@@ -131,6 +160,11 @@ def test_tags_from_path_plist_text(tmpdir):
         ('Pipfile.lock', {'text', 'json'}),
         ('mod/test.py', {'text', 'python'}),
         ('mod/Dockerfile', {'text', 'dockerfile'}),
+        ('Gemfile', {'text', 'ruby'}),
+        ('Gemfile.lock', {'text'}),
+        ('Jenkinsfile', {'text', 'groovy', 'jenkins'}),
+        ('build.jenkins', {'text', 'groovy', 'jenkins'}),
+        ('build.jenkinsfile', {'text', 'groovy', 'jenkins'}),
 
         # does not set binary / text
         ('f.plist', {'plist'}),
@@ -177,9 +211,9 @@ def test_tags_from_interpreter(interpreter, expected):
     (
         (b'hello world', True),
         (b'', True),
-        ('éóñəå  ⊂(◉‿◉)つ(ノ≥∇≤)ノ'.encode('utf8'), True),
-        (r'¯\_(ツ)_/¯'.encode('utf8'), True),
-        ('♪┏(・o･)┛♪┗ ( ･o･) ┓♪┏ ( ) ┛♪┗ (･o･ ) ┓♪'.encode('utf8'), True),
+        ('éóñəå  ⊂(◉‿◉)つ(ノ≥∇≤)ノ'.encode(), True),
+        (r'¯\_(ツ)_/¯'.encode(), True),
+        ('♪┏(・o･)┛♪┗ ( ･o･) ┓♪┏ ( ) ┛♪┗ (･o･ ) ┓♪'.encode(), True),
         ('éóñå'.encode('latin1'), True),
 
         (b'hello world\x00', False),
@@ -282,6 +316,8 @@ def test_file_is_text_does_not_exist(tmpdir):
         (b'\xf9\x93\x01\x42\xcd', ()),
         (b'#!\xf9\x93\x01\x42\xcd', ()),
         (b'#!\x00\x00\x00\x00', ()),
+        # shebang lines with multiple arguments
+        (b'#!/usr/bin/env -S python -u', ('python', '-u')),
     ),
 )
 def test_parse_shebang(s, expected):
@@ -306,6 +342,15 @@ def test_parse_shebang_from_file_simple(tmpdir):
     assert identify.parse_shebang_from_file(x.strpath) == ('python',)
 
 
+def test_parse_shebang_open_raises_einval(tmpdir):
+    x = tmpdir.join('f')
+    x.write('#!/usr/bin/env not-expected\n')
+    make_executable(x)
+    error = OSError(errno.EINVAL, f'Invalid argument {x}')
+    with mock.patch.object(builtins, 'open', side_effect=error):
+        assert identify.parse_shebang_from_file(x.strpath) == ()
+
+
 def make_executable(filename):
     original_mode = os.stat(filename).st_mode
     os.chmod(
